function [yhf,retal] = etalcalinsee(grocer_namey,varargin)
 
// PURPOSE: provide Insee's method for desaggegating time
// series with a high frequency indicator. The regression
// allows for:
//    (a) AR(1) or RW résiduals
//    (b) inclusion or exclusion of a constant
// ------------------------------------------------------------
// INPUT:
// * grocer_namey  = low frequency series
// * varargin = strings that can be
//  .'s=xx' frequency of the desired output and, accordingly,
//    of the indicators (default: 4)
//  .'alpha =xx' where w is significance level for the constant
//    default 0.1, that is 10%)
//  .'ny_prov =n' where n is the number of known annual data
//    that are ignored in the diseggregation process
//  .'mod=''rw''' if the residuals are supposed to be a RW or
//   'mod=''ar''' if the residuals are supposed to be an AR(1)
//  (default: model with maximum likelihood is chosen)
//  .'bench=''xxx''' where xxx is the name of the series the
//    user wants to compare the result with (for instance to
//    compare the method with another one)
//  .'noprint' if the user does not want to print the results
// ------------------------------------------------------------
// OUTPUT:
// * yhf = the disaggregated series
// * retal = a results tlist with:
//   - retal('meth') = 'Insee''s disaggregation'
//   - retal('y annu') = annual series
//   - retal('y estim') = annual transformed series (that is
//   differentiated if residuals follow a RW, levle if not)
//   - retal('x annu') = a (N x 1) vector of annual data
//   - retal('x estim') = a (N x 1) vector of transformed annual
//   data
//   - retal('x trim') = a (n x k) matrix of exogenous high
//   frequency indicators
//   - retal('nobs') = # of observations in the regression
//   - retal('resid estim') = a (n x 1) vector of regression
//   residuals
//   - retal('beta') = estimated coefficients of the annual
//   regression
//   - retal('tstat') = t stats
//   - retal('pvalue') = pvalue of the betas
//   - retal('prescte') = boolean indicating the presence or
//   absence of a constant in the regression
//   - retal('llike') = the log-likelihood
//   - retal('rho') = autcorrelation coefficient of residuals:
//  . if rho  = ]-1,1[ then the model is estimated in levels
//  . if rho = 1 then the model is estimated in differences
//   - retal('trend') = trend
//   . retal('trend') = 1 if retal('prescte') = %t and
//     rho = 1
//   . retal('trend') = 2 if retal('prescte') = %t and
//     rho ~= 1
//   . retal('trend') = 0 in other cases
//   - retal('prov y annu') = provisional value for y (and
//   therefore not used in estimation)
//   - retal('x annu aug') = (N x l) matrix of regressors,
//   including the constant or trend if necessary
//   - retal('x trim aug') = (n x l) matrix of regressors,
//   including the constant or trend if necessary
//   - retal('yhat annu') = (N x 1) adjusted annual y
//   - retal('resid annu') = (N x 1)annual residual
//   - retal('resid annu report') = annual residual extended
//   to the incomplete year with the estimated model
//   - retal('prov resid annu') = annual residual extended
//   using the last residual ("provisional year")
//   - retal('high freq yhat') = high frequency adjusted y
//   - retal('prov high freq yhat') = high frequency adjusted y
//   extended using the last residuals ("provisional years")
//   - retal('high freq resid') = high frequency residual in TS
//   form
//   - retal('prov high freq resid') = high frequency residual
//   extended using the last residuals ("provisional years") in
//   TS form
//   - retal('prests') = boolean indicating the presence or
//     absence of a time series in the regression
//   - retal('namey') = name of the y variable
//   - retal('namex') = name of the x variables
//   ------------------------------------------------------------
// Copyright E. Michaux (2004)
// http://dubois.ensae.net/grocer.html
 
// set defaults
grocer_s = 4
grocer_alpha = 0.1
grocer_affich  = 1
grocer_bench = 'no'
grocer_mod = 'no'
grocer_prt = %t
grocer_ny_prov = 1
grocer_opt= -1
 
grocer_nargin=length(varargin)
for grocer_i=grocer_nargin:-1:1
   if typeof(varargin(grocer_i)) == 'string' then
      str1=part(varargin(grocer_i),1:1)
      str3=part(varargin(grocer_i),1:3)
      str5=part(varargin(grocer_i),1:5)
      if str1 == 's' then
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i)=null()
      elseif str3 == 'mod' then
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i)=null()
      elseif str3 == 'opt' then
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i)=null()
      elseif str5 == 'bench' | str5 == 'alpha' then
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i)=null()
      elseif part(varargin(grocer_i),1:7) == 'ny_prov'
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i)=null()
      elseif varargin(grocer_i) == 'noprint' then
         varargin(grocer_i)=null()
         grocer_prt=%f
      end
   elseif typeof(varargin(grocer_i)) ~= 'constant' &...
   typeof(varargin(grocer_i)) ~= 'ts' then
      error('wrong type for entry number '+string(grocer_i+2))
   end
end
 
[y,x,namey,namexos,prests,boundsvarb,s,boundsy]=explo_agreg(-1,grocer_namey,varargin(:))
 
if s == 4 then
   slit='q'
elseif s == 12 then
   slit='m'
else
   error("bad frequency: only monthly and quarterly indicators allowed")
end
 
// provides the results from the regression of the vector y
// on the vector x
retal = etalcalinsee0(y,x,s,grocer_alpha,grocer_mod,grocer_ny_prov,grocer_opt)
 
// save the names, the bounds if the regression involves ts
retal(1)($+1) = 'prests'
retal(1)($+1) = 'namex'
retal(1)($+1) = 'namey'
 
retal('prests')=prests
retal('namex') =namexos
retal('namey') =namey
 
 
// transform the results into TS
boxd = date2num(boundsvarb(1))
boxf = date2num(boundsvarb(2))
yhf=tlist(['ts';'freq';'dates';'series'],retal('high freq'),[boxd:boxf]',retal('high freq yhat'))
retal('high freq yhat') = yhf
 
ts=yhf
ts('series')= retal('high freq resid')
retal('high freq resid') = ts
 
boyd =  boundsy(1)
retal('yhat annu') = reshape(retal('yhat annu'),boyd)
retal('resid annu') = reshape(retal('resid annu'),boyd)
retal('resid annu report') = reshape(retal('resid annu report'),boyd)
 
if grocer_prt then
   prtetalcal(retal)
 
   if grocer_bench ~= 'no'
 
     yhat = retal('high freq yhat')
 
     ng=winsid()  // recover the list of open graphic w
     if isempty(ng) then
        ng=1
     else
        ng = ng(size(ng,2)) +1   // add 1 to the last open window
     end
     dat1=boundsvarb(1)
     dat2=boundsvarb(2)
     ind_infra1=strindex(dat1,slit);
     execstr('year1='+part(dat1,1:ind_infra1-1))
     ind_infra2=strindex(dat2,slit);
     execstr('year2='+part(dat2,1:ind_infra2-1))
     execstr('infra2='+part(dat2,ind_infra2+1))
     scal=[string([year1:year2-1] .*. ones(1,s))+slit+string(ones(1,year2-year1) .*. [1:s]) , string(year2)+slit+string([1:infra2])]
     y=%nan*zeros((year2-year1)*s+infra2,2)
     if typeof(grocer_bench) == 'string' then
        execstr('grocer_bench='+grocer_bench)
     end
     d_bench=grocer_bench('dates')
     s_bench=grocer_bench('series')
     d_bench1=d_bench(1)
     d_bench2=d_bench($)
     if d_bench1 >= year1*s+1 then
        ind_y1=d_bench1-year1*s
        ind_bench1=1
     else
        ind_y1=1
        ind_bench1=year1*s-d_bench1+1
     end
 
     if d_bench2 <= year2*s+infra2 then
        ind_y2=d_bench2-year1*s
        ind_bench2=d_bench2-d_bench1+1
     else
        ind_y2=size(y,1)
        ind_bench2=year2*s+infra2-d_bench1+1
     end
 
     y(ind_y1:ind_y2,1)=s_bench(ind_bench1:ind_bench2)
     y(:,2)=yhat('series')
     pltseries0(y,[],'disaggregation of '+retal('namey'),scal,ng,'legend=[observed;disaggregated]','styleg=6')
 
   end
end
 
endfunction
